#include "gpx2shp.h"
#include "projects.h"
#include "geodesic.h"

void
checkEllpsUnit(char *unit)
{
/*
 * checks ellipse unit can be used by proj4
 */
#ifdef DEBUG
    printf("(checkEllpsUnit)\n");
#endif
    int             isOK = 0;
    struct PJ_ELLPS *el;	/* project.h of proj4 */
    for (el = pj_ellps; el->id; ++el) {
	if (!strcmp(el->id, unit)) {
	    isOK = 1;
	}
    }
    if (!isOK) {
	fputs
	    ("The ellipse argument is not correct or supported by libproj\n",
	     stderr);
	fputs("You can choose the argument from a list below.\n\n",
	      stderr);
	for (el = pj_ellps; el->id; el++) {
	    printf("%10s\t%s\n", el->id, el->name);
	}
	exit(0);
    }
}

double
checkLengthUnit(char *unit)
{
/*
 * checks length unit can be used by proj4 
 * then returns unit value to meter
 */
#ifdef DEBUG
    printf("(checkLengthUnit)\n");
#endif
    int             isOK = 0;
    double          to_meter = 0;
    struct PJ_UNITS *ut;	/* project.h of proj4 */
    for (ut = pj_units; ut->id; ut++) {
	if (!strcmp(ut->id, unit)) {
	    isOK = 1;
	    to_meter = atof(ut->to_meter);
	}
    }
    if (!isOK) {
	fputs
	    ("The length unit argument is not correct or supported by libproj.\n",
	     stderr);
	fputs("You can choose the argument from a list below.\n\n",
	      stderr);
	for (ut = pj_units; ut->id; ut++) {
	    printf("%s\t%s\n", ut->id, ut->name);
	}
	exit(0);
    }
    return to_meter;
}

int
checkTimeUnit(char *unit)
{
    char           *u[8] =
	{ "sec", "s", "min", "m", "hour", "h", "day", "d" };
    int             p[8] = { 1, 1, 60, 60, 3600, 3600, 86400, 86400 };
    int             i,
                    to_sec = 0;
    for (i = 0; i < 8; i++) {
	if (!strcmp(u[i], unit)) {
	    to_sec = p[i];
	}
    }
    if (!to_sec) {
	fputs("The time unit argument is not correct.\n", stderr);
	fputs("You can choose the argument from sec, min, hour or day.\n",
	      stderr);
	exit(0);
    }
    return to_sec;
}

int
getElementCount(xmlNodePtr cur)
{
/*
 * counts how many elements on the node.
 */
#ifdef DEBUG
    printf("(getElementCount)\n");
#endif
    int             c = 0;
    while (cur != NULL) {
	c++;
	cur = cur->next;
    }
    return c;
}

double
getTimeInterval(char *_t, char *t)
{
/*
 * Returns a time interval between _t and t.
 * The arguments should be "YYYY-MM-DDThh:mm:ssZ" (xml schema
 * datetime format without time zone) format.
 * I think there are more better ways. Do you have any idea?
 */
#ifdef DEBUG
    printf("(getTimeInterval)\n");
#endif
    double          ti;
    struct tm       _tt;
    struct tm       tt;
    time_t          _tmt,
                    tmt;
    sscanf(_t, "%d-%d-%dT%d:%d:%dZ", &_tt.tm_year, &_tt.tm_mon,
	   &_tt.tm_mday, &_tt.tm_hour, &_tt.tm_min, &_tt.tm_sec);
    _tt.tm_year -= 1900;
    _tt.tm_mon -= 1;
    sscanf(t, "%d-%d-%dT%d:%d:%d", &tt.tm_year, &tt.tm_mon, &tt.tm_mday,
	   &tt.tm_hour, &tt.tm_min, &tt.tm_sec);
    tt.tm_year -= 1900;
    tt.tm_mon -= 1;
    _tmt = mktime(&_tt);
    tmt = mktime(&tt);
    ti = difftime(tmt, _tmt);
    return ti;
}

double
getSpeed(double length, double ti, double to_meter, int to_sec)
{
/*
 * Culculates speed from length and time.
 */
#ifdef DEBUG
    printf("(getSpeed)\n");
#endif
    double          speed;
    if (!length || !ti)
	speed = 0;
    else
    /*
     * speed = (length / 1000) / (ti / 3600); 
     */
    /*
     * printf("%f, %f, %f, %i\n", length, to_meter, ti, to_sec); 
     */
	speed = (length / to_meter) / (ti / to_sec);
    return speed;
}

double
getDistanceCore(char *p1, char *l1, char *p2, char *l2)
{
/*
 * Culculates a geodesic length between two points
 * using geod_*.c
 */
#ifdef DEBUG
    printf("(getDistanceCore)\n");
#endif
    phi1 = dmstor(p1, &p1);
    lam1 = dmstor(l1, &l1);
    phi2 = dmstor(p2, &p2);
    lam2 = dmstor(l2, &l2);
    geod_inv();
    return geod_S;
}

double
getDistance(double _x, double _y, double x, double y)
{
/*
 * Converts arguments for distance culculation from
 * double to char.
 * I think it can be merged with getDistanceCore()
 * but I couldn't. I need to study...
 */
#ifdef DEBUG
    printf("(getDistance)\n");
#endif
    double          length;
    char            p1[17],
                    l1[17],
                    p2[17],
                    l2[17];
    sprintf(p1, "%f", _x);
    sprintf(l1, "%f", _y);
    sprintf(p2, "%f", x);
    sprintf(l2, "%f", y);
    length = getDistanceCore(p1, l1, p2, l2);
    return length;
}

void
closeFiles(g2sprop * pr)
{
/*
 * Closes all DBF and SHP files if they opened
 */
#ifdef DEBUG
    printf("(closeFiles)\n");
#endif
    if (pr->hDBF_wpt)
	DBFClose(pr->hDBF_wpt);
    if (pr->hDBF_trk)
	DBFClose(pr->hDBF_trk);
    if (pr->hDBF_trk_spt)
	DBFClose(pr->hDBF_trk_spt);
    if (pr->hDBF_trk_pnt)
	DBFClose(pr->hDBF_trk_pnt);
    if (pr->hDBF_rte)
	DBFClose(pr->hDBF_rte);
    if (pr->hDBF_rte_spt)
	DBFClose(pr->hDBF_rte_spt);
    if (pr->hDBF_rte_pnt)
	DBFClose(pr->hDBF_rte_pnt);
    if (pr->hSHP_wpt)
	SHPClose(pr->hSHP_wpt);
    if (pr->hSHP_trk)
	SHPClose(pr->hSHP_trk);
    if (pr->hSHP_trk_spt)
	SHPClose(pr->hSHP_trk_spt);
    if (pr->hSHP_trk_pnt)
	SHPClose(pr->hSHP_trk_pnt);
    if (pr->hSHP_rte)
	SHPClose(pr->hSHP_rte);
    if (pr->hSHP_rte_spt)
	SHPClose(pr->hSHP_rte_spt);
    if (pr->hSHP_rte_pnt)
	SHPClose(pr->hSHP_rte_pnt);
}

void
nullChange(char *attr)
{
/*
 * Change null char var to "-".
 */
    char            n[2];
    strcpy(n, "-");
    if (!strcmp(attr, "\0"))
	strcpy(attr, n);
}

void
failToWriteAttr(int iShape, int col, char *source)
{
#ifdef DEBUG
    printf("(failToWriteAttr)\n");
#endif
    printf("Fail to write a attribute in %s . shapeid:%i col:%i\n", source,
	   iShape, col);
}
